# include <cpput/assertcommon.h>
# include <cpput/message.h>
# include <cpput/translate.h>


namespace CppUT
{


CheckerResult 
pass()
{
   return CheckerResult();
}


CheckerResult 
fail()
{
   return fail( translate("Assertion Failed") );
}


CheckerResult 
fail( const LazyMessage &message )
{
   CheckerResult result;
   result.setFailed();
   result.message_ = message.makeMessage();
   return result;
}


CheckerResult 
checkTrue( bool shouldBeTrue,
           const LazyMessage &message )
{
   CheckerResult result;
   if ( !shouldBeTrue )
   {
      result.message_ = message.makeMessage();
      result.message_.insertAt( 0, translate( "expression did not evaluate to true." ) );
      result.setFailed();
   }
   return result;
}


CheckerResult 
checkFalse( bool shouldBeFalse,
            const LazyMessage &message )
{
   CheckerResult result;
   if ( shouldBeFalse )
   {
      result.message_ = message.makeMessage();
      result.message_.insertAt( 0, translate( "expression did not evaluate to false." ) );
      result.setFailed();
   }
   return result;
}


CheckerResult 
checkAssertionFail( bool assertionFailed,
                    const LazyMessage &message )
{
   CheckerResult result;
   if ( !assertionFailed )
   {
      result.message_ = Message( translate( "Assertion expression did not fail as expected." ) );
      result.message_.extend( message.makeMessage() );
      result.setFailed();
   }
   return result;
}


CheckerResult 
checkAssertionPass( bool assertionFailed,
                    const LazyMessage &message )
{
   CheckerResult result;
   if ( assertionFailed )
   {
      result.message_ = Message( translate( "Assertion expression did not pass as expected." ) );
      result.message_.extend( message.makeMessage() );
      result.setFailed();
   }
   return result;
}


void
buildEqualityFailedMessage( CheckerResult &result,
                            const std::string &expected,
                            const std::string &actual,
                            const LazyMessage &message )
{
   result.setFailed();
   result.message_ = message.makeMessage();
   result.message_.add( translate( "Equality assertion failed." ) );
   result.message_.add( translate( "Expected: " ) + expected );
   result.message_.add( translate( "Actual  : " ) + actual );
}


void
buildUnequalityFailedMessage( CheckerResult &result,
                              const std::string &expected,
                              const std::string &actual,
                              const LazyMessage &message )
{
   result.setFailed();
   result.message_ = message.makeMessage();
   result.message_.add( translate( "Unequality assertion failed (expected & actual should be different)." ) );
   result.message_.add( translate( "Expected: " ) + expected );
   result.message_.add( translate( "Actual  : " ) + actual );
}


CheckerResult 
checkDoubleEquals( double expected, 
                   double actual, 
                   double tolerance, 
                   const LazyMessage &message )
{
   CheckerResult result;
   double diff = expected - actual;
   if ( diff < 0 )
      diff = -diff;
   if ( diff > tolerance )
   {
      result.setFailed();
      result.message_ = message.makeMessage();
      result.message_.add( translate( "Double equality assertion failed." ) );
      result.message_.add( translate( "Expected: " ) + CppTL::toString( expected ) );
      result.message_.add( translate( "Actual  : " ) + CppTL::toString( actual ) );
      result.message_.add( translate( "Tolerance  : " ) + CppTL::toString( tolerance ) );
      result.message_.add( translate( "Actual difference  : " ) + CppTL::toString( diff ) );
   }
   return result;
}


void 
skipCurrentTest()
{
   TestStatus &status = TestInfo::threadInstance().testStatus();
   if ( status.hasPassed() )
      status.setStatus( TestStatus::skipped );
   throw SkipTestException();
}


} // namespace CppUT

